@@ -36,6 +36,20 @@
 
 #include "debug.h"
 
+/* Helper function to escape underscores in labels
+ * before putting them in menu items */
+static char *
+escape_label_for_menu (const char *name)
+{
+	char *new, **a;
+
+	a = g_strsplit (name, "_", -1);
+	new = g_strjoinv ("__", a);
+	g_strfreev (a);
+
+	return new;
+}
+
 /* ISO-639 helpers */
 static GHashTable *lang_table;
 
@@ -227,29 +241,27 @@ add_lang_action (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 		int lang_id, GSList **group)
 {
 	const char *full_lang;
+	char *label;
 	char *name;
 	GtkAction *action;
 
 	full_lang = totem_lang_get_full (lang);
-
+	label = escape_label_for_menu (full_lang ? full_lang : lang);
 	name = g_strdup_printf ("%s-%d", prefix, lang_id);
 
 	action = g_object_new (GTK_TYPE_RADIO_ACTION,
-			"name", name,
-			"label", full_lang ? full_lang : lang,
-			"value", lang_id,
-			NULL);
-
-	gtk_radio_action_set_group (GTK_RADIO_ACTION (action),
-			*group);
+			       "name", name,
+			       "label", label,
+			       "value", lang_id,
+			       NULL);
+	g_free (label);
 
+	gtk_radio_action_set_group (GTK_RADIO_ACTION (action), *group);
 	*group = gtk_radio_action_get_group (GTK_RADIO_ACTION (action));
-
 	gtk_action_group_add_action (action_group, action);
 	g_object_unref (action);
-
 	gtk_ui_manager_add_ui (totem->ui_manager, ui_id,
-			path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
+			       path, name, name, GTK_UI_MANAGER_MENUITEM, FALSE);
 	g_free (name);
 
 	return action;
@@ -261,9 +273,11 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 		gboolean is_lang)
 {
 	GtkAction *action = NULL;
-	int i;
+	unsigned int i, *hash_value;
 	GList *l;
 	GSList *group = NULL;
+	GHashTable *lookup;
+	char *action_data;
 
 	if (is_lang == FALSE) {
 		add_lang_action (totem, action_group, ui_id, path, prefix,
@@ -274,14 +288,27 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 			_("Auto"), -1, &group);
 
 	i = 0;
+	lookup = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 
 	for (l = list; l != NULL; l = l->next)
 	{
+		hash_value = g_hash_table_lookup (lookup, l->data);
+		if (hash_value == NULL) {
+			action_data = g_strdup (l->data);
+			g_hash_table_insert (lookup, l->data, GINT_TO_POINTER (1));
+		} else {
+			guint num = GPOINTER_TO_INT (hash_value);
+			action_data = g_strdup_printf ("%s #%u", (char *)l->data, num + 1);
+			g_hash_table_replace (lookup, l->data, GINT_TO_POINTER (num + 1));
+		}
+
 		add_lang_action (totem, action_group, ui_id, path, prefix,
-				l->data, i, &group);
-		i++;
+				 action_data, i, &group);
+ 		i++;
 	}
 
+	g_hash_table_destroy (lookup);
+
 	return action;
 }
 
@@ -550,6 +577,8 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 		GtkAction     *action;
 		char          *action_name;
 		char          *label;
+		char          *escaped_label;
+		char          *label_trunc;
 		const char    *uri;
 
 		info = (GtkRecentInfo *) l->data;
@@ -559,40 +588,43 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 
 		action_name = g_strdup_printf ("recent-file%u", i++);
 		uri = gtk_recent_info_get_uri (info);
-		if (strstr (uri, "file:///") == NULL)
-			label = g_strdup_printf ("_%d.  %s", n_items + 1, uri);
-		else {
-			label = g_strdup_printf ("_%d.  %s", n_items + 1,
-				gtk_recent_info_get_display_name (info));
-		}
 
-		/* Truncate the menu item if it's too long */
-		char *label_trunc = totem_str_middle_truncate (label, TOTEM_MAX_RECENT_ITEM_LEN);
-		g_free (label);
+		/* Munge the URI for display */
+		if (g_str_has_prefix (uri, "file:///") == FALSE) {
+			label_trunc = totem_str_middle_truncate (uri, TOTEM_MAX_RECENT_ITEM_LEN);
+		} else {
+			label_trunc = totem_str_middle_truncate
+				(gtk_recent_info_get_display_name (info),
+				 TOTEM_MAX_RECENT_ITEM_LEN);
+		}
+		escaped_label = escape_label_for_menu (label_trunc);
+		g_free (label_trunc);
+		label = g_strdup_printf ("_%d.  %s", n_items + 1, escaped_label);
+		g_free (escaped_label);
 
 		action = g_object_new (GTK_TYPE_ACTION,
-			       "name", action_name,
-			       "label", label_trunc,
-			       NULL);
+				       "name", action_name,
+				       "label", label,
+				       NULL);
 
 		g_object_set_data_full (G_OBJECT (action), "recent-info",
-				gtk_recent_info_ref (info),
-				(GDestroyNotify) gtk_recent_info_unref);
+					gtk_recent_info_ref (info),
+					(GDestroyNotify) gtk_recent_info_unref);
 		g_signal_connect (G_OBJECT (action), "activate",
-			  	G_CALLBACK (on_recent_file_item_activated),
-				totem);
+				  G_CALLBACK (on_recent_file_item_activated),
+				  totem);
 
 		gtk_action_group_add_action (totem->recent_action_group,
-				action);
+					     action);
 		g_object_unref (action);
 
 		gtk_ui_manager_add_ui (totem->ui_manager, totem->recent_ui_id,
-			       "/tmw-menubar/movie/recent-placeholder",
-			       label_trunc, action_name, GTK_UI_MANAGER_MENUITEM,
-			       FALSE);
+				       "/tmw-menubar/movie/recent-placeholder",
+				       label, action_name, GTK_UI_MANAGER_MENUITEM,
+				       FALSE);
 
 		g_free (action_name);
-		g_free (label_trunc);
+		g_free (label);
 
 		if (++n_items == 5)
 			break;
@@ -701,18 +733,6 @@ my_gnome_vfs_volume_get_mount_path (GnomeVFSVolume *volume)
 	return path;
 }
 
-static char *
-escape_device_name_for_menu (const char *name)
-{
-	char *new, **a;
-
-	a = g_strsplit (name, "_", -1);
-	new = g_strjoinv ("__", a);
-	g_strfreev (a);
-
-	return new;
-}
-
 static void
 add_device_to_menu (GObject *device, guint position, Totem *totem)
 {
@@ -749,7 +769,7 @@ add_device_to_menu (GObject *device, guint position, Totem *totem)
 		&gnome_vfs_drive_get_device_path);
 
 	g_strstrip (name);
-	escaped_name = escape_device_name_for_menu (name);
+	escaped_name = escape_label_for_menu (name);
 	g_free (name);
 	label = g_strdup_printf (_("Play Disc '%s'"), escaped_name);
 	g_free (escaped_name);
@@ -1183,7 +1203,7 @@ static const GtkActionEntry entries[] = {
 
 	{ "edit-menu", NULL, N_("_Edit") },
 	{ "take-screenshot", "camera-photo", N_("Take _Screenshot..."), "<control>S", N_("Take a screenshot"), G_CALLBACK (take_screenshot_action_callback) },
-	{ "clear-playlist", NULL, N_("Clear Playlist"), NULL, N_("Clear Playlist"), G_CALLBACK (clear_playlist_action_callback) },
+	{ "clear-playlist", NULL, N_("_Clear Playlist"), NULL, N_("Clear playlist"), G_CALLBACK (clear_playlist_action_callback) },
 	{ "preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, NULL, G_CALLBACK (preferences_action_callback) },
 
 	{ "view-menu", NULL, N_("_View") },
