@@ -30,11 +30,11 @@
 
 G_BEGIN_DECLS
 
-#define GTK_TYPE_PLAYLIST            (totem_playlist_get_type ())
-#define TOTEM_PLAYLIST(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_PLAYLIST, TotemPlaylist))
-#define TOTEM_PLAYLIST_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_PLAYLIST, TotemPlaylistClass))
-#define GTK_IS_PLAYLIST(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_PLAYLIST))
-#define GTK_IS_PLAYLIST_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_PLAYLIST))
+#define TOTEM_TYPE_PLAYLIST            (totem_playlist_get_type ())
+#define TOTEM_PLAYLIST(obj)            (GTK_CHECK_CAST ((obj), TOTEM_TYPE_PLAYLIST, TotemPlaylist))
+#define TOTEM_PLAYLIST_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TOTEM_TYPE_PLAYLIST, TotemPlaylistClass))
+#define TOTEM_IS_PLAYLIST(obj)         (GTK_CHECK_TYPE ((obj), TOTEM_TYPE_PLAYLIST))
+#define TOTEM_IS_PLAYLIST_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TOTEM_TYPE_PLAYLIST))
 
 typedef enum {
 	TOTEM_PLAYLIST_DIRECTION_NEXT,
@@ -79,7 +79,7 @@ void totem_playlist_save_current_playlist_ext (TotemPlaylist *playlist,
 
 /* totem_playlist_clear doesn't emit the current_removed signal, even if it does
  * because the caller should know what to do after it's done with clearing */
-void       totem_playlist_clear (TotemPlaylist *playlist);
+gboolean   totem_playlist_clear (TotemPlaylist *playlist);
 void       totem_playlist_clear_with_prefix (TotemPlaylist *playlist,
 					     const char *prefix);
 void       totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
