@@ -35,9 +35,10 @@
 #include <libnautilus-extension/nautilus-extension-types.h>
 #include <libnautilus-extension/nautilus-property-page-provider.h>
 
-#include "data/totem-mime-types.h"
+#include "totem-mime-types.h"
 
 static GType tpp_type = 0;
+static gboolean backend_inited = FALSE;
 static void property_page_provider_iface_init
 	(NautilusPropertyPageProviderIface *iface);
 static GList *totem_properties_get_pages
@@ -107,10 +108,15 @@ totem_properties_get_pages (NautilusPropertyPageProvider *provider,
 	if (found == FALSE)
 		goto end;
 
-	/* okay, make the page */
+	/* okay, make the page, init'ing the backend first if necessary */
+	if (backend_inited == FALSE) {
+		bacon_video_widget_init_backend (NULL, NULL);
+		backend_inited = TRUE;
+	}
 	uri = nautilus_file_info_get_uri (file);
 	label = gtk_label_new (_("Audio/Video"));
 	page = totem_properties_view_new (uri, label);
+	gtk_container_set_border_width (GTK_CONTAINER (page), 6);
 	property_page = nautilus_property_page_new ("video-properties",
 			label, page);
 
@@ -129,7 +135,6 @@ nautilus_module_initialize (GTypeModule *module)
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-	bacon_video_widget_init_backend (NULL, NULL);
 	totem_properties_plugin_register_type (module);
 	totem_properties_view_register_type (module);
 }
