@@ -28,14 +28,14 @@
 
 #include "config.h"
 
-#ifndef HAVE_GTK_ONLY
-#include <gnome.h>
-#include "totem-gromit.h"
-#else
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
+
+#ifndef HAVE_GTK_ONLY
+#include <gnome.h>
+#include "totem-gromit.h"
 #endif /* !HAVE_GTK_ONLY */
 
 #include <string.h>
@@ -57,6 +57,7 @@
 #include "totem-screenshot.h"
 #include "totem-sidebar.h"
 #include "totem-menu.h"
+#include "totem-missing-plugins.h"
 #include "totem-options.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
@@ -81,6 +82,8 @@
 #define ZOOM_UPPER 200
 #define ZOOM_RESET 100
 #define ZOOM_LOWER 10
+#define ZOOM_DISABLE (ZOOM_LOWER - 1)
+#define ZOOM_ENABLE (ZOOM_UPPER + 1)
 
 #define FULLSCREEN_POPUP_TIMEOUT 5 * 1000
 
@@ -106,6 +109,7 @@ static void playlist_changed_cb (GtkWidget *playlist, Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
 static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
 static void popup_timeout_remove (Totem *totem);
+static gint totem_compare_recent_stream_items (GtkRecentInfo *a, GtkRecentInfo *b);
 
 static void
 long_action (void)
@@ -195,6 +199,12 @@ totem_action_exit (Totem *totem)
 	if (totem == NULL)
 		exit (0);
 
+#ifdef HAVE_REMOTE
+	if (totem->remote != NULL) {
+		g_object_unref (G_OBJECT (totem->remote));
+	}
+#endif /* HAVE_REMOTE */
+
 	if (totem->win != NULL) {
 		gtk_widget_hide (totem->win);
 		display = gtk_widget_get_display (totem->win);
@@ -358,7 +368,7 @@ totem_action_play (Totem *totem)
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
 	if (totem_is_fullscreen (totem) != FALSE)
-		totem_scrsaver_disable (totem->scr);
+		totem_scrsaver_set_state (totem->scr, !retval);
 
 	if (retval == FALSE)
 	{
@@ -412,6 +422,40 @@ totem_action_set_mrl_and_play (Totem *totem, const char *mrl)
 		totem_action_play (totem);
 }
 
+static gboolean
+totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
+{
+	GSList *filenames;
+	gboolean playlist_modified;
+
+	filenames = totem_add_files (GTK_WINDOW (totem->win), path);
+
+	if (filenames == NULL)
+		return FALSE;
+
+	playlist_modified = totem_action_open_files_list (totem,
+			filenames);
+
+	if (playlist_modified == FALSE) {
+		g_slist_foreach (filenames, (GFunc) g_free, NULL);
+		g_slist_free (filenames);
+		return FALSE;
+	}
+
+	g_slist_foreach (filenames, (GFunc) g_free, NULL);
+	g_slist_free (filenames);
+
+	if (play != FALSE) {
+		char *mrl;
+
+		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		totem_action_set_mrl_and_play (totem, mrl);
+		g_free (mrl);
+	}
+
+	return TRUE;
+}
+
 static gboolean
 totem_action_load_media (Totem *totem, MediaType type)
 {
@@ -467,12 +511,7 @@ totem_action_load_media_device (Totem *totem, const char *device)
 		case MEDIA_TYPE_DATA:
 			/* Set default location to the mountpoint of
 			 * this device */
-			{
-				char *s;
-				s = totem_action_open_dialog (totem, url, FALSE);
-				retval = (s != NULL);
-				g_free (s);
-			}
+			retval = totem_action_open_dialog (totem, url, FALSE);
 			break;
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
@@ -648,44 +687,144 @@ totem_action_fullscreen (Totem *totem, gboolean state)
 void
 totem_action_open (Totem *totem)
 {
-	static char *path = NULL;
-	char *new_path;
+	totem_action_open_dialog (totem, NULL, TRUE);
+}
 
-	new_path = totem_action_open_dialog (totem, path, TRUE);
-	if (new_path != NULL) {
-		g_free (path);
-		path = new_path;
+static gint
+totem_compare_recent_stream_items (GtkRecentInfo *a, GtkRecentInfo *b)
+{
+	gboolean has_totem_a, has_totem_b;
+
+	has_totem_a = gtk_recent_info_has_group (a, "TotemStreams");
+	has_totem_b = gtk_recent_info_has_group (b, "TotemStreams");
+
+	if (has_totem_a && has_totem_b) {
+		time_t time_a, time_b;
+
+		time_a = gtk_recent_info_get_modified (a);
+		time_b = gtk_recent_info_get_modified (b);
+
+		return (time_b - time_a);
+	} else if (has_totem_a) {
+		return -1;
+	} else if (has_totem_b) {
+		return 1;
 	}
+
+	return 0;
+}
+
+static char *
+totem_open_location_set_from_clipboard (Totem *totem)
+{
+	GtkClipboard *clipboard;
+	gchar *clipboard_content;
+
+	/* Initialize the clipboard and get its content */
+	clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (totem->win), GDK_SELECTION_CLIPBOARD);
+	clipboard_content = gtk_clipboard_wait_for_text (clipboard);
+
+	/* Check clipboard for "://". If it exists, return it */
+	if (clipboard_content != NULL && strcmp (clipboard_content, "") != 0)
+	{
+		if (g_strrstr (clipboard_content, "://") != NULL)
+			return clipboard_content;
+	}
+
+	g_free (clipboard_content);
+	return NULL;
+}
+
+static gboolean
+totem_open_location_match (GtkEntryCompletion *completion, const gchar *key, GtkTreeIter *iter, gpointer user_data)
+{
+	/* Substring-match key against uri */
+	char *uri, *match;
+
+	g_return_val_if_fail (key != NULL, FALSE);
+	gtk_tree_model_get (user_data, iter, 0, &uri, -1);
+	g_return_val_if_fail (uri != NULL, FALSE);
+	match = strstr (uri, key);
+	g_free (uri);
+
+	return (match != NULL);
 }
 
-//FIXME move the URI to a separate widget
 void
 totem_action_open_location (Totem *totem)
 {
 	GladeXML *glade;
-	char *mrl;
-	GtkWidget *dialog, *entry, *gentry;
+	char *mrl, *clipboard_location;
+	GtkWidget *dialog, *entry;
 	int response;
 	const char *filenames[2];
+	GtkEntryCompletion *completion;
+	GtkTreeModel *model;
+	GList *recent_items;
 
 	glade = totem_interface_load ("uri.glade", _("Open Location..."),
 			FALSE, GTK_WINDOW (totem->win));
 	if (glade == NULL)
 		return;
 
+	/* Get item from clipboard to fill entry in Open Location... dialog */
+	clipboard_location = totem_open_location_set_from_clipboard (totem);
+	if (clipboard_location != NULL && strcmp (clipboard_location, "") != 0)
+		gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (glade, "uri")), clipboard_location);
+	g_free (clipboard_location);
+
 	dialog = glade_xml_get_widget (glade, "open_uri_dialog");
 	entry = glade_xml_get_widget (glade, "uri");
-	gentry = glade_xml_get_widget (glade, "uri_list");
+
+	completion = gtk_entry_completion_new();
+	model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
+	gtk_entry_set_completion (GTK_ENTRY (entry), completion);
+
+	/* Add items in Totem's GtkRecentManager to the uri_list GtkEntry's GtkEntryCompletion */
+	recent_items = gtk_recent_manager_get_items (totem->recent_manager);
+
+	if (recent_items != NULL)
+	{
+		GList *p;
+		GtkTreeIter iter;
+
+		recent_items = g_list_sort (recent_items, (GCompareFunc) totem_compare_recent_stream_items);
+
+		for (p = recent_items; p != NULL; p = p->next)
+		{
+			if (!gtk_recent_info_has_group ((GtkRecentInfo *) p->data, "TotemStreams"))
+				continue;
+
+			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, gtk_recent_info_get_uri ((GtkRecentInfo *) p->data), -1);
+			gtk_recent_info_unref ((GtkRecentInfo *) p->data);
+		}
+	}
+
+	g_list_free (recent_items);
+
+	gtk_entry_completion_set_model (completion, model);
+	gtk_entry_completion_set_text_column (completion, 0);
+	gtk_entry_completion_set_match_func (completion, (GtkEntryCompletionMatchFunc) totem_open_location_match, model, NULL);
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	if (response == GTK_RESPONSE_OK)
 	{
-		const char *uri;
+		char *uri;
+
+		uri = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
-		uri = gtk_entry_get_text (GTK_ENTRY (entry));
 		if (uri != NULL && strcmp (uri, "") != 0)
 		{
+			if (g_strrstr (uri, "://") == NULL)
+			{
+				char *tmp;
+				tmp = g_strconcat ("http://", uri, NULL);
+				g_free (uri);
+				uri = tmp;
+			}
+
 			filenames[0] = uri;
 			filenames[1] = NULL;
 			totem_action_open_files (totem, (char **) filenames);
@@ -693,11 +832,8 @@ totem_action_open_location (Totem *totem)
 			mrl = totem_playlist_get_current_mrl (totem->playlist);
 			totem_action_set_mrl_and_play (totem, mrl);
 			g_free (mrl);
-#ifndef HAVE_GTK_ONLY
-			gnome_entry_append_history (GNOME_ENTRY (gentry),
-					TRUE, uri);
-#endif /* !HAVE_GTK_ONLY */
 		}
+		g_free (uri);
 	}
 
 	gtk_widget_destroy (dialog);
@@ -739,7 +875,7 @@ totem_action_take_screenshot (Totem *totem)
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
-	gdk_pixbuf_unref (pixbuf);
+	g_object_unref (pixbuf);
 }
 
 static char *
@@ -881,6 +1017,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		/* Take a screenshot */
 		totem_action_set_sensitivity ("take-screenshot", FALSE);
 
+		/* Clear the playlist */
+		totem_action_set_sensitivity ("clear-playlist", FALSE);
+
 		/* Set the logo */
 		bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
 		gtk_widget_set_sensitive (GTK_WIDGET (totem->properties),
@@ -917,6 +1056,10 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 
 		/* Take a screenshot */
 		totem_action_set_sensitivity ("take-screenshot", retval);
+
+		/* Clear the playlist */
+		totem_action_set_sensitivity ("clear-playlist", retval);
+
 		gtk_widget_set_sensitive
 			(GTK_WIDGET (totem->properties), retval);
 
@@ -1053,23 +1196,33 @@ static void
 totem_action_zoom (Totem *totem, int zoom)
 {
 	GtkAction *action;
+	gboolean zoom_reset, zoom_in, zoom_out;
 
-	if (zoom < ZOOM_LOWER || zoom > ZOOM_UPPER)
+	if (zoom == ZOOM_ENABLE)
+		zoom = bacon_video_widget_get_zoom (totem->bvw);
+
+	if (zoom == ZOOM_DISABLE) {
+		zoom_reset = zoom_in = zoom_out = FALSE;
+	} else if (zoom < ZOOM_LOWER || zoom > ZOOM_UPPER) {
 		return;
-
-	bacon_video_widget_set_zoom (totem->bvw, zoom);
+	} else {
+		bacon_video_widget_set_zoom (totem->bvw, zoom);
+		zoom_reset = (zoom != ZOOM_RESET);
+		zoom_out = zoom != ZOOM_LOWER;
+		zoom_in = zoom != ZOOM_UPPER;
+	}
 
 	action = gtk_action_group_get_action (totem->zoom_action_group,
 			"zoom-in");
-	gtk_action_set_sensitive (action, zoom != ZOOM_UPPER);
+	gtk_action_set_sensitive (action, zoom_in);
 
 	action = gtk_action_group_get_action (totem->zoom_action_group,
 			"zoom-out");
-	gtk_action_set_sensitive (action, zoom != ZOOM_LOWER);
+	gtk_action_set_sensitive (action, zoom_out);
 
 	action = gtk_action_group_get_action (totem->zoom_action_group,
 			"zoom-reset");
-	gtk_action_set_sensitive (action, zoom != ZOOM_RESET);
+	gtk_action_set_sensitive (action, zoom_reset);
 }
 
 void
@@ -1518,15 +1671,16 @@ update_volume_sliders (Totem *totem)
 	totem->prev_volume = volume;
 }
 
-static int
-gui_update_cb (Totem *totem)
+static void
+property_notify_cb (BaconVideoWidget *bvw, GParamSpec *spec, Totem *totem)
 {
-	if (totem->bvw == NULL)
-		return TRUE;
-
-	update_volume_sliders (totem);
-
-	return TRUE;
+	if (strcmp ("volume", spec->name) == 0) {
+		update_volume_sliders (totem);
+	} else if (strcmp ("logo-mode", spec->name) == 0) {
+		gboolean enabled;
+		enabled = bacon_video_widget_get_logo_mode (totem->bvw);
+		totem_action_zoom (totem, enabled ? ZOOM_DISABLE : ZOOM_ENABLE);
+	}
 }
 
 static gboolean
@@ -1637,10 +1791,14 @@ static gboolean
 totem_action_open_files_list (Totem *totem, GSList *list)
 {
 	GSList *l;
-	gboolean cleared = FALSE;
+	gboolean changed;
+	gboolean cleared;
+
+	changed = FALSE;
+	cleared = FALSE;
 
 	if (list == NULL)
-		return cleared;
+		return changed;
 
 	for (l = list ; l != NULL; l = l->next)
 	{
@@ -1673,18 +1831,21 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				g_signal_handlers_disconnect_by_func
 					(G_OBJECT (totem->playlist),
 					 playlist_changed_cb, totem);
-				totem_playlist_clear (totem->playlist);
+				changed = totem_playlist_clear (totem->playlist);
 				bacon_video_widget_close (totem->bvw);
 				cleared = TRUE;
 			}
 
 			if (totem_is_block_device (filename) != FALSE) {
 				totem_action_load_media_device (totem, data);
-			} else if (strstr (filename, "cdda:/") != NULL) {
+				changed = TRUE;
+			} else if (g_str_has_prefix (filename, "cdda:/") != FALSE) {
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
+				changed = TRUE;
 			} else if (totem_playlist_add_mrl (totem->playlist,
 						filename, NULL) != FALSE) {
 				totem_action_add_recent (totem, filename);
+				changed = TRUE;
 			}
 		}
 
@@ -1699,43 +1860,7 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				totem);
 	}
 
-	return cleared;
-}
-
-char *
-totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
-{
-	GSList *filenames;
-	char *new_path;
-	gboolean playlist_modified;
-
-	filenames = totem_add_files (GTK_WINDOW (totem->win),
-			path, &new_path);
-
-	if (filenames == NULL)
-		return NULL;
-
-	playlist_modified = totem_action_open_files_list (totem,
-			filenames);
-
-	if (playlist_modified == FALSE) {
-		g_slist_foreach (filenames, (GFunc) g_free, NULL);
-		g_slist_free (filenames);
-		return NULL;
-	}
-
-	g_slist_foreach (filenames, (GFunc) g_free, NULL);
-	g_slist_free (filenames);
-
-	if (play != FALSE) {
-		char *mrl;
-
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
-		g_free (mrl);
-	}
-
-	return new_path;
+	return changed;
 }
 
 static void
@@ -1775,6 +1900,7 @@ void
 totem_action_skip_to (Totem *totem)
 {
 	char *filename;
+	TotemSkipto **skipto;
 
 	if (totem->seekable == FALSE)
 		return;
@@ -1793,8 +1919,11 @@ totem_action_skip_to (Totem *totem)
 			G_CALLBACK (gtk_widget_destroy), NULL);
 	g_signal_connect (G_OBJECT (totem->skipto), "response",
 			G_CALLBACK (commit_hide_skip_to), totem);
+
+	skipto = &totem->skipto;
 	g_object_add_weak_pointer (G_OBJECT (totem->skipto),
-			(gpointer *)&totem->skipto);
+				   (gpointer *) skipto);
+
 	gtk_window_set_transient_for (GTK_WINDOW (totem->skipto),
 			GTK_WINDOW (totem->win));
 	gtk_widget_show (GTK_WIDGET (totem->skipto));
@@ -2086,6 +2215,8 @@ playlist_changed_cb (GtkWidget *playlist, Totem *totem)
 	if (mrl == NULL)
 		return;
 
+	//FIXME we shouldn't compare the 2 URLs
+	// http://bugzilla.gnome.org/show_bug.cgi?id=364311
 	if (totem->mrl == NULL
 			|| (totem->mrl != NULL && mrl != NULL
 			&& strcmp (totem->mrl, mrl) != 0))
@@ -2315,8 +2446,13 @@ static gboolean
 on_video_button_press_event (BaconVideoWidget *bvw, GdkEventButton *event,
 		Totem *totem)
 {
-	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
-	{
+	if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
+		totem_action_fullscreen_toggle(totem);
+		return TRUE;
+	} else if (event->type == GDK_BUTTON_PRESS && event->button == 2) {
+		totem_action_play_pause(totem);
+		return TRUE;
+	} else if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
 		totem_action_menu_popup (totem, event->button);
 		return TRUE;
 	}
@@ -2742,6 +2878,18 @@ on_volume_scroll_event (GtkWidget *win, GdkEventScroll *event, Totem *totem)
 	return totem_action_handle_volume_scroll (totem, event->direction);
 }
 
+#ifdef HAVE_MEDIA_PLAYER_KEYS
+static gboolean
+on_window_focus_in_event (GtkWidget *win, GdkEventFocus *event, Totem *totem)
+{
+	if (totem->remote != NULL) {
+		totem_remote_window_activated (totem->remote);
+	}
+
+	return FALSE;
+}
+#endif
+
 static void
 update_media_menu_items (Totem *totem)
 {
@@ -2758,7 +2906,6 @@ update_media_menu_items (Totem *totem)
 	totem_action_set_sensitivity ("next-angle", playing);
 
 	playing = totem_is_media (totem->mrl);
-
 	totem_action_set_sensitivity ("eject", playing);
 }
 
@@ -2899,11 +3046,6 @@ totem_callback_connect (Totem *totem)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 		totem_playlist_get_shuffle (totem->playlist));
 
-#ifdef HAVE_GTK_ONLY
-	action = gtk_action_group_get_action (totem->main_action_group, "contents");
-	gtk_action_set_visible (action, FALSE);
-#endif /* HAVE_GTK_ONLY */
-
 	/* Controls */
 	box = glade_xml_get_widget (totem->xml, "tmw_buttons_hbox");
 
@@ -3055,8 +3197,10 @@ totem_callback_connect (Totem *totem)
 	totem_action_set_sensitivity ("next-chapter", FALSE);
 	totem_action_set_sensitivity ("previous-chapter", FALSE);
 
-	/* Update the UI */
-	g_timeout_add (600, (GSourceFunc) gui_update_cb, totem);
+#ifdef HAVE_MEDIA_PLAYER_KEYS
+	g_signal_connect (G_OBJECT(totem->win), "focus-in-event",
+			G_CALLBACK (on_window_focus_in_event), totem);
+#endif
 }
 
 static void
@@ -3095,6 +3239,7 @@ video_widget_create (Totem *totem)
 {
 	GError *err = NULL;
 	GtkWidget *container;
+	BaconVideoWidget **bvw;
 
 	totem->scr = totem_scrsaver_new ();
 
@@ -3120,7 +3265,7 @@ video_widget_create (Totem *totem)
 			"motion-notify-event",
 			G_CALLBACK (on_video_motion_notify_event),
 			totem);
-	g_signal_connect (G_OBJECT (totem->bvw),
+	g_signal_connect_after (G_OBJECT (totem->bvw),
 			"button-press-event",
 			G_CALLBACK (on_video_button_press_event),
 			totem);
@@ -3157,6 +3302,8 @@ video_widget_create (Totem *totem)
 			G_CALLBACK (on_error_event),
 			totem);
 
+	totem_missing_plugins_setup (totem);
+
 	container = glade_xml_get_widget (totem->xml, "tmw_bvw_vbox");
 	gtk_container_add (GTK_CONTAINER (container),
 			GTK_WIDGET (totem->bvw));
@@ -3182,14 +3329,17 @@ video_widget_create (Totem *totem)
 			source_table, G_N_ELEMENTS (source_table),
 			GDK_ACTION_LINK);
 
+	bvw = &(totem->bvw);
 	g_object_add_weak_pointer (G_OBJECT (totem->bvw),
-			(void**)&(totem->bvw));
+				   (gpointer *) bvw);
 
 	gtk_widget_show (GTK_WIDGET (totem->bvw));
 
 	bacon_video_widget_set_volume (totem->bvw,
 			gconf_client_get_int (totem->gc,
 				GCONF_PREFIX"/volume", NULL));
+	g_signal_connect (G_OBJECT (totem->bvw), "notify",
+			G_CALLBACK (property_notify_cb), totem);
 	update_volume_sliders (totem);
 }
 
@@ -3234,6 +3384,13 @@ main (int argc, char **argv)
 {
 	Totem *totem;
 	GConfClient *gc;
+#ifndef HAVE_GTK_ONLY
+	GnomeProgram *program;
+#else
+	GError *error = NULL;
+#endif
+	GOptionContext *context;
+	GOptionGroup *baconoptiongroup;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -3250,14 +3407,23 @@ main (int argc, char **argv)
 
 	g_thread_init (NULL);
 
+	/* Handle command line arguments */
+	context = g_option_context_new (_("- Play movies and songs"));
+	baconoptiongroup = bacon_video_widget_get_option_group();
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, baconoptiongroup);
+
 #ifdef HAVE_GTK_ONLY
-	gtk_init (&argc, &argv);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	if (g_option_context_parse (context, &argc, &argv, &error) == FALSE) {
+		totem_action_error_and_exit (_("Totem could not parse the command-line options"), error->message, NULL);
+	}
 #else
-	gnome_program_init ("totem", VERSION,
+	program = gnome_program_init (PACKAGE, VERSION,
 			LIBGNOMEUI_MODULE,
 			argc, argv,
 			GNOME_PARAM_APP_DATADIR, DATADIR,
-			GNOME_PARAM_POPT_TABLE, totem_options_get_options (),
+			GNOME_PARAM_GOPTION_CONTEXT, context,
 			GNOME_PARAM_NONE);
 #endif /* HAVE_GTK_ONLY */
 
@@ -3282,13 +3448,13 @@ main (int argc, char **argv)
 	totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
 	if (bacon_message_connection_get_is_server (totem->conn) == FALSE)
 	{
-		totem_options_process_for_server (totem->conn, argc, argv);
+		totem_options_process_for_server (totem->conn, &optionstate);
 		bacon_message_connection_free (totem->conn);
 		g_free (totem);
 		gdk_notify_startup_complete ();
 		exit (0);
 	} else {
-		totem_options_process_early (gc, argc, argv);
+		totem_options_process_early (gc, &optionstate);
 	}
 
 	/* Init totem itself */
@@ -3365,10 +3531,12 @@ main (int argc, char **argv)
 	totem_setup_preferences (totem);
 
 	/* Command-line handling */
-	if (totem_options_process_late (totem, &argc, &argv) != FALSE)
+	totem_options_process_late (totem, &optionstate);
+
+	if (totem->session_restored != FALSE)
 	{
-		totem_session_restore (totem, argv);
-	} else if (argc >= 1 && totem_action_open_files (totem, argv)) {
+		totem_session_restore (totem, optionstate.filenames);
+	} else if (optionstate.filenames != NULL && totem_action_open_files (totem, optionstate.filenames)) {
 		totem_action_play_pause (totem);
 	} else {
 		totem_action_set_mrl (totem, NULL);
@@ -3390,5 +3558,9 @@ main (int argc, char **argv)
 
 	gtk_main ();
 
+#ifndef HAVE_GTK_ONLY
+	/* Will destroy GOption allocated data automatically */
+	g_object_unref (program);
+#endif	
 	return 0;
 }
